// Technische Richtlinie FA205 
// Bibliothek:       communication.c
// Controller:       alle 8051 (at89c5131)
//     I2C und RS232 Routinen
// Version:          1.0
// erstellt am:      30.9.2015
// letzte nderung:  26.10.2015
// Autor:            Rahm

#include "communication.h"

// ****************************************************************************
// I2C- Routinen zur Ansteuerung eins I2C-Slaves
// I2C-Bus-Funktionen i2c_init, i2c_start, i2c_stop, i2c_write, i2c_read
// ****************************************************************************

// **** Zeitverzgerung zur Verlangsamung der Datenbertragungsrate ***********
// **** i=2 bis i=100 whlen je nach I2C-IC und Pull-Up-Widerstand
void i2c_delay(void) 
{
  volatile uint8_t i;	         // i nicht rausoptimieren (volatile)!!
  for (i=5;i!=0;i--);
}

// ****************************************************************************
// ****** Initialiserung I2C-Bus **********************************************
void i2c_init (void)
{									
  I2C_PORT |= (1<<SDA);                  // Leitungen in den Grundzustand High
  i2c_delay();
  I2C_PORT |= (1<<SCL);
  i2c_delay();
}
	 
// ****** Startbedingung I2C-Bus **********************************************
void i2c_start (void)
{
  I2C_PORT |= (1<<SDA);                  // Grundzustand
  I2C_PORT |= (1<<SCL);
  i2c_delay();
  I2C_PORT &= ~(1<<SDA);                 // Wechsel SDA von 1 -> 0 whrend SCL = 1	
  i2c_delay();
  I2C_PORT &= ~(1<<SCL);	
  i2c_delay();
}

//****** Stoppbedingung I2C-Bus ***********************************************************
void i2c_stop (void)
{
  I2C_PORT &= ~(1<<SDA);                 // SDA = 0;
  I2C_PORT |= (1<<SCL);                  // SCL = 1;
  i2c_delay();
  // Wechsel von SDA 0 -> 1 whrend SCL = 1
  I2C_PORT |= (1<<SDA);                  // SDA = 1;
  i2c_delay();
}

//*****************************************************************************************
// * Byte ausgeben an I2C-Bus , Rckgabewert = ack = ACK/NACK 	
// ****************************************************************************************
uint8_t i2c_write (uint8_t value)
{
  uint8_t z;                             // Zhler 																	
  uint8_t ack;                           // Acknoledge-Bit															
		
  for (z = 8; z != 0; z --)              // Zhler: serielle Ausgabe von 8 Bit								
  {	
    if ((value & 0x80) == 0x80)          // Ausgabe: MSB first
      I2C_PORT |= (1<<SDA);              // SDA = 1;
    else	
      I2C_PORT &= ~(1<<SDA);             // SDA = 0;
		
    value <<=1;                          // nchstes Bit!	
		
    i2c_delay();										
    I2C_PORT |= (1<<SCL);                // SCL = 1 Daten sind gltig			
    i2c_delay();
    I2C_PORT &= ~(1<<SCL);               // SCL = 0 Datenausabe beendet													
    i2c_delay();
  }

  I2C_PORT |= (1<<SDA);                  // SDA = 1 Leitung freigeben fr Acknoledge-Bit								
  i2c_delay();
  I2C_PORT |= (1<<SCL);                  // SCL = 1 Slave kann besttigen													
  i2c_delay();                           // warten

  ack = (I2C_PORT & (1<<SDA))>>SDA;
		
  i2c_delay();
  I2C_PORT &= ~(1<<SCL);                 // SCL = 0 Slave soll Ackn-Ausgabe beenden	
  i2c_delay();									

  return ack;                            // Acknoledge-Bit ist Rckgabewert der Funktion	
                                         // ack = 0 bedeutet Slave hat "verstanden" !!!!!!						
}

//*****************************************************************************************
// * Byte einlesen vom I2C-Bus.
// ****************************************************************************************
uint8_t i2c_read (uint8_t ack)
{
  uint8_t z, value = 0;
	
  I2C_PORT |= (1<<SDA);                   // SDA = 1  Leitung freigeben (high) fr Daten						

  for (z = 8; z != 0; z--)                // Zhler: serielles Einlesen von 8 Bit					
  {
    I2C_PORT |= (1<<SCL);                 // SCL = 1  Daten sind gltig												
    i2c_delay();

    value <<= 1;
    value |= (uint8_t) ((I2C_PORT & (1<<SDA))>>SDA);    //SDA   Datenbit in Puffer

    i2c_delay();
    I2C_PORT &= ~(1<<SCL);                // SCL = 0  Daten lesen beendet	
    i2c_delay();										
  }
  
  if (ack == ACK)  I2C_PORT &= ~(1<<SDA); // SDA = 0  (= Acknowledge)
  else             I2C_PORT |= (1<<SDA);  // SDA = 1  (= Not Acknowledge)
  
  i2c_delay();		
  I2C_PORT |= (1<<SCL);                   // SCL = 1  Ackn. gltig													
  i2c_delay();
  I2C_PORT &= ~(1<<SCL);                  // SCL = 0  Ackn. beendet													
  i2c_delay();
  I2C_PORT &= ~(1<<SDA);                  // SDA = 0  Leitung SDA vorbereiten fr Stoppbed.				
  i2c_delay();

  return ( value );                       // eingelesenes Byte = Rckgabewert													
}

// ****************************************************************************
// RS232-Routinen zur Kommunikation mit PC-Terminal
// 
// ****************************************************************************

// Serielle Schnittstelle initialisieren
void rs232_init ( void )
{
  PCON |= 0x80;             // 0b10000000 -> SMOD1 = 1
  SCON  = 0x52;             // 0b01010010 -> UART in mode 1 (8 bit), REN=1
                            //         TI = 1 !! (TI-Abfrage vor senden!)
  // Baudrate-Generator-Reloadwert
  // ==> RS_Reload = 256 - fclk / 32 / BAUD
  //     RS_Reload = 217  (9600Bd, 12MHz), (19200Bd, 24MHz)
  //     RS_Reload = 178  (9600Bd, 24MHz)
  #define RS_RELOAD   0x100 - F_CPU / 32 / BAUD
  
  BRL    = RS_RELOAD;
  BDRCON = 0x1e;            // 0b00011110 -> Baud rate generator run
}

uint8_t rs232_get ( void )
{
  if (!RI)  return ('\0');  // Wenn kein Zeichen im Puffer, dann 0 zurckgeben!
  RI = 0;                   // Empfangsflag lschen
  return SBUF;              // Zeichen zurckliefern
}

void rs232_put ( uint8_t value )
{
  while (!TI);              // Warten bis Sendebereit! (TI mit 1 initialisieren!)
  TI = 0;                   // Sendeflag lschen
  SBUF = value;
}

void rs232_print ( uint8_t *text )
{
  while (*text != '\0')
    rs232_put(*text++);
	delay_ms(500);
}
